#ifndef __HVCHANNELLT
#define __HVCHANNELLT
#include <map>
#include "hvchannelinterface.h"
/*! \file hvchannellt.h
    \brief  The class connect the correct HV channel to the detector (ID )
*/

/*! \class HvChannelLt
    \brief The class connect the correct HV channel to the device ID

    To use the object the HvBias interface should provide all necesary functions
    When instantiate the object an device  ID should be given. The class
    constructor request the data base for the information about the type of 
    HV bias supply is connected to the device  ID . Also it request the information
    to which particular channel of the Hv supply the detector is connected.
    It creates a corresponding HV object with the HvBiasInterface as interface class too.
    This object take care of the realization of the abstract functions.
    see the hvchannellt.cxx file for implementatoin code 

    \author W. Beaumont Universtity Antwerpen
    \version 1.0
    \date 2001 05 28
*/ 
class HvChannelLt :  public HVChannelInterface 
{  private:
   int   fType; 
   HVChannelInterface *HvBias ;
   int fDeviceId; 
   float fIGain;
   struct RegHvCh { 
     HVChannelInterface* hv;
     int nrreg;
   };
   static std::map <int,RegHvCh*> fRegHv;
   bool fRefferenceToChannel;
  
  
 public: 
   
   HvChannelLt( int deviceID );
 
   virtual ~HvChannelLt();
   //! 
   bool isRefference(){ return fRefferenceToChannel;}

   //!  gives the HV Volt in volts  since the last readout 
   virtual float  getHvVolt( )
   { return HvBias->getHvVolt( );}

   virtual float  getHvCurrent( )    { 
    return fIGain*HvBias->getHvCurrent( );
   }
     
     
   //! returns the device ID of the channel
   virtual int getId(){return fDeviceId;}

   //! get the current HV ramp setting of the device (since last readout) in [V/s]
   virtual float getHvRampUp()
     { return HvBias-> getHvRampUp();}

  //! get the current HV ramp down  setting of the device (since last readout) in [V/s]
   virtual float getHvRampDown()
     { return HvBias-> getHvRampDown();}
   //!  get the current HV current limit  setting of the device since last readout result is in uA    
   virtual float getHvCurrentLimit()
     {  return fIGain*(HvBias-> getHvCurrentLimit()) ;}

   //!  get the status of the HV unit 
   /*!  unit depending like HV on, overflow ... 
     voltage set point reached , ramping ... 
     returns -1 if fails or function is not implemented for the chosen device
   */
   virtual int getHvStatus()
     { return HvBias-> getHvStatus() ;}
   
   //!  get the maximum voltage that is allowed 
   /*!  if return value is zero it could be this function is not available in the current device
    */
   virtual float getHvVoltLimit()  { return HvBias-> getHvVoltLimit();}
    
   virtual int getV0() { return HvBias->getV0();}
   
   //! get the limit voltage that is allowed for this channel  this number is set by the setHvSwLimit
   virtual float getHvSwVoltLimit()
     { return HvBias->getHvSwVoltLimit();}   
   
   //!set the software hv limit  return the limit 
   virtual int  setHvSwVoltLimit(float newvalue)
     { return HvBias->setHvSwVoltLimit( newvalue);}
   
   //! set the HV value to  newvalue in volts return 0 if success
   /*!  returns -1 if fails or function is not implemented for the chosen device
     returns -2 if the requested value is higher than the setHvSwLimit ;
   */

   virtual int  setHvVolt(float newvalue)
     { return HvBias-> setHvVolt( newvalue);}
   
   //!  set the ramp voltage to newvalue [V/s]
   /*!  returns -1 if fails or function is not implemented for the chosen device
    */
   virtual int setHvRampUp(float newvalue )
     { return HvBias-> setHvRampUp( newvalue );}
   //!  set the ramp voltage to newvalue [V/s]
   /*!  returns -1 if fails or function is not implemented for the chosen device
    */
   virtual int setHvRampDown(float newvalue )
     { return HvBias-> setHvRampDown( newvalue );}
   //! set the current limit to newvalue in uA
   /*!  returns -1 if fails or function is not implemented for the chosen device
    */  
   virtual int setHvCurrentLimit(float newvalue)
     { return HvBias->setHvCurrentLimit( newvalue/fIGain);}

   //! set the voltage limit to newvalue in V
   /*!   returns -1 if fails or function is not implemented for the chosen device
    */
   virtual int setHvVoltLimit(float newvalue)
     { return HvBias->setHvVoltLimit(newvalue);}
   /** turn on the Hv device */
   virtual void turnOn(){HvBias->turnOn();}
   /** turn off the Hv device */
   virtual void turnOff(){HvBias->turnOff();}  
   
   virtual void  lock()
   {  HvBias-> lock();}

   virtual void  unlock()
     {  HvBias-> unlock();     }
  /** Request DAQ to change parameter */
  virtual void justDoIt() {  HvBias->justDoIt();}
  /** Release of channel to the DAQ */
  virtual void  release(){  HvBias->release();}
 
  /** Polling routine avoiding pause/signal problems TO BE USED! */
  virtual int wait(){ return HvBias->wait();}

  virtual void setReadOnly(bool t){HvBias->setReadOnly(t);}

   
   //!< send info about the class to cout only for debugging
   /*!< displays the values of the 
     - current voltage
     - ramp voltage
     - current current
     - current limmit
   */
  virtual void Info();
 
};

#endif 
