#ifndef __CONVERSIONS
#define __CONVERSIONS

#include <cstring>
#include <cmath>
#include <vector>
#include <cstdio>
#include <iostream>
#include "calexchange.h"

#define TCZERO 273.15  // T celcius zero 




/** \file CalibrationValues.h
    \brief definition of the calibration getters and setters 
    
    For the moment it is a collection of all conversion classes
    These classes should get its own include files and perhaps code files where needed.
    at june 2002 the basic class CalExchange is moved to its own file 
    
*/ 

/*  Adc Cal values  ( singel ended unipolair 0 -5 V 
    ADC's 0 -15 , 32 - 47
    input       ADC value     
      0 V              7
      12.3mV           17
      2.439            2008
      4.97             4091    (DAC value 4082) 
*/




/** the name space for the HV bias channel class and HV conversion class
    namespace HvConversionDef */
namespace HvConversionDef 
{   enum knownHvTypes {DUMMYDEVICEID=-222,DUMMY=0,  HvCAEN_A33 , HvSTRUCK_NHQ201 ,HvCAEN_N470, HvCAEN_A434,HvCAEN_2527,HvCAEN_527,HvDIM };   

   typedef struct 
      {	float CalVoltage;/// is the real  measured voltgage
	float  CalVoltageSetting ; /// The voltage setting 
	float Gain; // gain range setting for V 
	
	float ZeroVoltage ; /// the voltage when set to zero
	float Calcurrent;  /// the measured current
	float CurrentOut;  /// the current the device gives out 
	float CurrentGain;  /// the gain to get A
	float ZeroCurrent;  /// the output indication when there is no current
      } HvCalValues ; 
}

/** 
    \class  HvConversion 
    \author W. Beaumont
    \date May  2001
    \version 1.0


    \brief converts the values from a device to Volts in [V] or  to cyrrents [A]  (for getCurrent)

    Used to convert the value comming from one of the devices to a current in amperes.
    Most devices will give already a Volt or Amperes. This conversion function is meant to adjust the 
    device values with a factor obtained from calibrations. 
   
    When instantiated the  object CalExchange<CurrentConvDef::CurrentCalValues> should be provided
    to fill the calibration values. The CalExchange class also provides the methods to change the calibration 
    values and identifacation of the sensor

    The  class  is used by LtParser to get  the information read from the lt.xml file.
    
    The class is not in use at the oment
    
*/
class HvConversion  :  public CalExchange <HvConversionDef::HvCalValues>  

{ 
   // definition 
  public:
  HvConversion(CalExchange<HvConversionDef::HvCalValues> CE):CalExchange<HvConversionDef::HvCalValues>(CE)
      {};  

  /// get the corrected Hv for the moment only gain correction to get V
  float getHv ( float HVin ) { return  fCV.Gain * HVin;}
  float getCurrent( float Cin){  return  fCV.CurrentGain * Cin;}
} ;    

/** the name space for the TemperartureChannel class  sensor and TempConversion class

 */


namespace TempConverDef 

{  /// know types of temperature sensors 
   enum TempSensType { DUMMYDEVICEID=-222,DUMMY=0, AD590 , NTC , 
		       PTC , PT100, LINEAR,  AD590DUMMY, TRHXSENS, 
		       TRHXNTC, TEMPCTRL, TEMPSPEC,NTCAD4717 };
   typedef struct 
      {	float CalTemp;// is the temperature @ offset_Adc
	int CalAdcValue; // Adc Value @ cal termperature
	int OpenAdcValue; // Adc Value if no temp sensor is connected (power supply for NTC's )
	float Gain; // gain = C / ADC count
                // Vrange / I bias for thermistors 
	
	float Beta;
      } TempCalValues ; 
   float NtcResistor2temperature( float R , float R0 , float T0, float beta);
   float Temperature2NtcResitor(float T, float R0,float T0, float beta);
}
     
/** 
    \class  TempConversion 
    \author W. Beaumont
    \date January   2001
    \version 1.1
     \date   September 2001
    \version 1.2 
    \brief This class takes care to convert ADC value to a temperature [C] 
    
    
    
    Only the AD590 / ADC combination is tested 
     When instantiated the  object CalExchange<TempConverDef::TempCalValues>  should be provided
    to fill the calibration values. The CalExchange class also provides the methods to change the calibration 
    values and identifacation of the sensor
    
    For the moment it has to define the type. Perhaps lateron each type will get its own class.
    For the moment there is only a class definition will all inline code.
*/ 

class TempConversion  :  public CalExchange <TempConverDef::TempCalValues>  

{ float fBaseValue; //can be resistor by example for NTC  
  public :
    TempConversion():CalExchange<TempConverDef::TempCalValues>()
      {fCV.CalTemp=0;	fCV.CalAdcValue=1; 	fCV.OpenAdcValue=2;
      fCV.Gain=0; 	 fCV.Beta=1; fBaseValue=0;
      } 
    TempConversion(CalExchange<TempConverDef::TempCalValues> CE):CalExchange<TempConverDef::TempCalValues>(CE)
      {};  
  /// the conversion from ADC counts to Celsius  using the CalExchagne / TmepCalValues 
  /*! in case of the THRX temperature sensor the temperature is already converted by the 
      routines that came with the THRX system 
      it is here to cope with eventual calibration corrections 
  */
    float getBaseValue(){ return fBaseValue;}
  float    Adc2Tconversion (float  AdcValue);
  inline void setCalTemperatue ( float Temp) { fCV.CalTemp = Temp;}
  inline void setCalAdcValue   ( int  Adc ) { fCV.CalAdcValue = Adc;}
  inline void setTemperature2AdcGain (float T2AGain) { fCV.Gain = T2AGain;}
  inline void PrintTCVvalues(float TestAdcValue) 
    {  std::cout <<printIdValues(std::cout);
       printf("type: %d   \n",  fCId.fType);
       printf(" CalTemp \t %f\n", fCV.CalTemp);  
       printf(" CalAdcValue \t %d\n", fCV.CalAdcValue);  
       printf(" OpenAdcValue \t %d\n", fCV.OpenAdcValue);  
       printf(" Gain \t %f\n", fCV.Gain);  
       printf(" beta \t %f\n", fCV.Beta);
       printf(" TestAdcValue \t %f\n", TestAdcValue);  
       printf(" BaseValue\t %f\n",fBaseValue);
       printf(" Result Temp \t %f \n", Adc2Tconversion(TestAdcValue));
     }
  };



namespace CurrentConvDef {
  /// the different type of current meters 
  enum CurrentType { DUMMYDEVICEID=-222,DUMMY=0,ADCELECTROMETER , HVMODULE,ADC, I2CCURRENTCHANNEL,AD4717, CURRENTSPEC=100};
  /// values to be used by the current conversion 
 typedef struct {
    ///  the current  @ offset_Adc in[A]
    float CalCurrent;
    /// Adc Value @ cal current 
    float  CalAdcValue; 
    /// gain = A / ADC count
    float Gain; 
   } CurrentCalValues ; 
}

/** 
    \class CurrentConversion
    \author W. Beaumont
    \date May  2001
    \version 1.0


    \brief converts the values from a device to Amperes

    Used to convert the value comming from one of the devices to a current in amperes.
    At this moment only the electro meter connected to an ADC is implemented.
    
    When instantiated the  object CalExchange<CurrentConvDef::CurrentCalValues> should be provided
    to fill the calibration values. The CalExchange class also provides the methods to change the calibration 
    values and identifacation of the sensor

    The  class  is used by LtParser to get  the information read from the lt.xml file.
    
    The class is inherited by the CurrentChannel class.
    
*/

class CurrentConversion:  public CalExchange <CurrentConvDef::CurrentCalValues> 
 {
   // definition 
 public: 
   
 private:
  
 public :
   /// empty initialization 
   CurrentConversion(){ fCV.CalCurrent=0 ;  fCV.CalAdcValue=1;   fCV.Gain=0; }
   /// class  CalExchange<CurrentConvDef::CurrentCalValues>  that holds the calibration values
   inline  CurrentConversion(CalExchange<CurrentConvDef::CurrentCalValues> c ) : 
                  CalExchange<CurrentConvDef::CurrentCalValues> (c)
      {    }

 /// the conversion from ADC counts to ampere given the values in the CalExchange structure
 inline float    Adc2Cconversion (float  AdcValue)
    { // assumption current 0 uA = 0 Adc
    return  (float)(AdcValue-fCV.CalAdcValue) * fCV.Gain + fCV.CalCurrent;
  }

 /// use this function only for debug reasons. 
  inline void printHCCValues(float  TestAdcValue) 
    { std::cout << printIdValues(std::cout);
      printf(" CalHv \t %f\n", fCV.CalCurrent);  
      printf(" CalAdcValue \t %f\n", fCV.CalAdcValue);  
      printf(" Gain \t %f\n", fCV.Gain);  
      printf(" TestAdcValue \t %f\n", TestAdcValue);  
      printf(" Result HVcurrent \t %f \n", Adc2Cconversion(TestAdcValue));
    }
} ;

namespace DioChannelDef
{ enum DIOCHANNELTYPES {  DUMMYDEVICEID=-222,DUMMY=0, SERIAL , CIODIO , 
			  AD4717DIO,PCF8574DIO,ACTIVE};
}


namespace BoxCharacteristicsDef
{  int const MAXTEMPCTRLS=2;
   enum BoxType { DUMMY,  STANDARD ,DIMCTRL, WIENCTRL ,  COLDBOX,TEMPCTRLDUMMY  };
  /// structure that contain the characteristics of the cold box
  typedef struct {
    /// the weight of the box not in use 
    float fWeigth ;
    /// the heat capacity of the box in J/ C used to calculate the temperature control response
    float fHeatCapacity ;

    /// the heat leakage  in W/C  used to calculate the temperature control response
    float fLeakage ; 

    /// the first order time response on a power change 
    float fHeatTimeConst;

    /// the overall loop gain of the system 
    float  fLoopGain ;
    
    /// not in use
    float  fDiffGain;
    
    /// not in use 
    float fFullPowerMargin;
 
    /// the box follow number 
    int fBoxNr;

    /// the max temperature difference used for calculations ( if more it is truncated)
    float fMaxDeltaTemperature ; 
   
    /// the absolute minimum temperature allowed in the box. If this is reached cooling will stop
    float fMinTemperature ; 
    
    /// the absolute maximum temperature in the box. When this is reached temperature control will be switched off
    float fMaxTemperature ; 
    
    /// the temperature of the cooling water 
    float fWaterTemperature ; 
 
   /// the temperature channel ID to probe the temperature for the temperature controler
    int fTemperatureId;  

    /// the current control device 
    int fCurrentChannelId;

    /// the power conversion ID that control the cooling power 
    int fPowerConversionId; 
    /// the humidity control Id ;
    int fHumidityCtrlId; 
    /// the water temperature Id
    int fWaterTemperatureId;
    /// the hot side temperature Id
    int fHotSideTemperatureId; 
    /// the number of temperature control loops
    std::vector <int> fTempCtrlId;
  } BoxCharValues;

 
};

/*  \class  BoxCharacteristics
    \author W. Beaumont  
    \date May 2001
    \version 1.00

    \brief this class contains the information about the cold box like heat leakage etc.
    
    The function is used by LtParser to get store the information read from the lt.xml file.
    The temperature controler reads the values to calculate the para meters for the control loop
*/ 
class BoxCharacteristics: public CalExchange <BoxCharacteristicsDef::BoxCharValues>
{  public:
  //using 
  //BoxCharacteristicsDef:: BoxCharValues;
  BoxCharacteristics (CalExchange <BoxCharacteristicsDef::BoxCharValues> * c ): 
    CalExchange <BoxCharacteristicsDef::BoxCharValues> (*c) 
             {}
 
  void setCalValues (BoxCharacteristicsDef::BoxCharValues   *BCV_in)
    {  CalExchange <BoxCharacteristicsDef::BoxCharValues>::setCalValues( BCV_in);
    fCV.fHeatTimeConst = fCV.fHeatCapacity / fCV.fLeakage ; 
    // overwrites settings in set BCV values 
    }  
};


namespace HumidityCtrlDef
{ enum HumidityCtrlType {DUMMY, STANDARD,PROBEONLY };
 typedef struct { 
   /// the device number of the humidity control
   int fHumidityId;
   /// the device number of the gasflow device
   int fGasFlowId;
   /// the minimum humidity level -1 is not control 
   float fMinHumidity;
   /// the maximum humidity during stable operation
   float  fMaxHumidity;
   /// the maximum humidity before cool down
   float fCoolDownHumidity;
   /// max gas flow that can be set 
   float fMaxGasFlow;
 }HumidityCtrlValues;
 //std::string getTypeName(){return std::string("HumidityCtrl");}
};

namespace GasFlowDef
{ enum GasFlowType {DUMMY, DAC, SERIAL };
 typedef struct { 
   /// gain / for polarity use 1 or -1 
   int fGain;
   /// the Dac value for which the output is maximal 
   unsigned short fMaxDac ;  
 }GasFlowValues;
};

namespace PowerConversionDef
{   enum PowerConversinType {DUMMY, DAC , SERIAL , POL=100 , POLCOM=200  };
  /// this structure holds the values for the conversion from a power number in Watt to a DAC value  
  typedef struct {
    /// the know power in Watt at a given DAC value (CalDacValue)
    float CalPower; 
    /// see CalPower 
    unsigned short int CalDacValue;
    
    /// the lineair factor between the ouput power ([W])  and the DAC value 
    float  Gain; // in W/count

    /// the maximum value for the DAC allowed 
    unsigned short int   MaxDac ;

    /// the bit value for a positive output in case powerchannelpol is used
    int PosOutput;
    /// the bit value for a positive output in case powerchannelpol is used
    int NegOutput;
    
    /// int the ID number of the dio channel  (this is not a device number !!
    int DioChannelId;


    } PowerCalValues;
}

class PowerConversion :  public CalExchange < PowerConversionDef:: PowerCalValues>
{ 
 public:
 
  PowerConversion(CalExchange < PowerConversionDef:: PowerCalValues>  c  )
    :CalExchange < PowerConversionDef:: PowerCalValues>(c) {  }
 
  unsigned short  Power2Dacconversion(float PValue);
  
  float Dac2Powerconversion ( unsigned short DValue);
  
  void PrintPCVvalues(float  TestPowerValue) ;
           
};



namespace HumidityConversionDef

{  /// this structure holds the values for the conversion from a ADC count to a humidity values in % 

enum HumiditySensType { DUMMY,HUMIRELADC, TRHXSENS ,HUMSPECIAL  };
 typedef struct {
     float CalHumidity;
    float CalAdcValue;
    float CalVoltageRef;
    float Gain;
    float VRefGain;
    float VRefAdcValue;
   float Offset; 
    } HumidityCalValues;
}


/** 
    \class HumidityConversion
    \author W. Beaumont
    \date May  2001
    \version 1.0


    \brief converts a ADC value to a humidty value in %


    Used to convert the value comming from one of the devices to a humidiyt in % 
    At this moment only the Humirel  connected to an ADC is implemented.
    
    When instantiated the  object CalExchange<CurrentConvDef::HumidityCalValues> should be provided
    to fill the calibration values. The CalExchange class also provides the methods to change the calibration 
    values and identifacation of the sensor

    The  class  is used by LtParser to get  the information read from the lt.xml file.
    
    The class is inherited by the HumidiytChannel class.
    
    
    

 
     To make the conversion two values are needed beside the calibraion values  :
     The ac
     Vout  =  Vref * (0.2354 + 0.00474 * Rh)
     Vref = 1/fVRefGain * AdcGain * AdcValue(ref channel)
     Adc gain = 5/4069 = 1.2207  E-3 V /bit 
     Vout =  AdcValue * AdcGain / VrefGain
     AdcValue = AdcValue (refchan)/VrefGain   *  (0.2354 + 0.00474 * Rh)
     Rh = (AdcValue / fVrefAdcValue ) -0.2354 / 0.00474 

     new formular
     refADC  = 1/fVrefGain * AdcGain *AdcValue(refChannel) 
     assume ADC gain = 1 

     general function:
     
     Vout = Vsupply /5  * (gain Rh + offset)
     Vout/Adcgain      = ADCref@5/AdcGain  /5  * (gain *Rh +offset);
     Adc= AdcRef@5  /5 *(gain *Rh+offset)
     Vsupply = Vref/vrefgain  AdcRef@5 = ADCref/vrefgain
     Rh = ((5*Adc /AdcRef@5) -offset )/gain
 

   
*/

class HumidityConversion : public CalExchange < HumidityConversionDef::HumidityCalValues>
{ 

 protected: 
  
  void setVref ( float AdcValue)
    { fCV.VRefAdcValue =  AdcValue/fCV.VRefGain;  } 
 public:
 
  HumidityConversion(){ fCV.CalHumidity=1;
   fCV.CalAdcValue=1;
   fCV.CalVoltageRef=5;
   fCV.Gain=0;
   fCV.VRefGain=1;
   fCV.VRefAdcValue=0;
   fCV.Offset=0;
  }
 HumidityConversion(CalExchange < HumidityConversionDef::HumidityCalValues> CE ): CalExchange < HumidityConversionDef::HumidityCalValues>(CE) { }
 
 float   Adc2HumidityConversion(float HAdcValue);
 
 inline void printHCvalues(float  TestHumidityAdcValue) 
                     {  std::cout << printIdValues(std::cout);
                       printf(" CalHumidity \t %f\n", fCV.CalHumidity);  
                       printf(" CalAdcValue \t %f\n", fCV.CalAdcValue);  
                       printf(" Gain \t %f\n", fCV.Gain);  
		       printf(" Vref is \t %f\n", fCV.VRefAdcValue);
                       printf(" TestHumidityalue \t %f\n", TestHumidityAdcValue);  
		       printf(" Result \t %f \n", Adc2HumidityConversion(TestHumidityAdcValue));
		     }


};

#endif // __CONVERSIONS

#ifndef __CALIBRATIONVALUES
#define __CALIBRATIONVALUES

/** \brief the getters of the calibration values for the moment hard coded
    
    At the moment each type of sensor has his own method. I don't see an other way for the moment
    The user of the sensor has not to know about this class he can get ( and later on set) the 
    calibration data via the  CalExchange class what is inherited by the sensor class.

 */


class CalValues
{  //definitions
  public : 



//  inline CalValues ( char* filename )
/*
  opens the calibration file  and stores this in the different 
    defined sturctures  as long as there is no protocol is  defined 
   for the configuration files this will not be coded  only simple 
   functions to have the interface  
    return zero when filename  is read succesful  else -1
    should only once read the file and store the data
    later objects should only check the structure at 
*/

/** \brief The getter for the temperature calibration parameters.
 */ 
  int  GetTemperartureCalParameter(  TempConversion* Tch  )
  // TmpChId the temparture channel 
  // TempCalVal the structure to store the calibration values
  // returns 0 if o.k.
  // should look up the TmpChId and return the values 
    { // TempConversion::
      using  TempConverDef::AD590;
      using  TempConverDef::NTC;
       TempConverDef::TempCalValues* value = new  TempConverDef::TempCalValues; 
      int ChId = Tch->getId();
      const float   AD590gain = -0.003179 ; 
      switch (ChId)
	{ case 4 :  // hardcoded now to be a NTC 
	         
	          value->Gain=   100E3 ; // assume  5V   with bias 50uA;
	          value->CalTemp=25.0;
                  value->CalAdcValue=16*3740; // guess 
		  
		  value->Beta = 3350 ;
	   break;
	
	  case 1:  // assume AD590 as default 
	    value->Gain=   AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 76.29 mV  ( this is not the reslution !! 
                   // 1 C  == 314.57   ADC counts  >> gain = -0.003179 
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2906; // calculated 
		   
		   value->Beta = 0 ;
	      break;
	  
	  case 3:  // assume AD590 as default 
	           value->Gain=  AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2987; // calculated 
		   
		   value->Beta = 0 ;
	    break; 
	  case 5:  // assume AD590 as default 
	           value->Gain=   AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 16*4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2988; // calculated 
		  
		   value->Beta = 0 ;
	    break; 
	  case 7:  // assume AD590 as default 
	           value->Gain=  AD590gain;
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue=16* 2985; // calculated 
		  
		   value->Beta = 0 ;
	    break; 

	  case 9:  // assume AD590 as default 
	           value->Gain=   AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2885; // calculated 
		   
		   value->Beta = 0 ;
	    break; 
	  case 11:  
	           value->Gain=  AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2945 ; // calculated 
		   
		   value->Beta = 0 ;
		  break;   

	  case 13:  
	           value->Gain=   AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2992; // calculated 
 		   
		   value->Beta = 0 ;
		  break;   
	  case 15:  
	           value->Gain=   AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=22.5;
		   value->CalAdcValue= 16*2912; // calculated 
 		   
		   value->Beta = 0 ;
		break;   

	  case 21:  
	           value->Gain=   -0.050833;
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=23.5;
		   value->CalAdcValue= 16*3002; // calculated 
 	  
		   value->Beta = 0 ;
		break; 
		   
	   default:  // assume AD590 as default 
	           value->Gain=    AD590gain; 
                   // 24K * 1uA == 0.024  V / C  drop !! if termparture increase
                   // 1 ADC = 5 / 4096 = 1.22 mV 
                   // 1 C  == 19,67  ADC counts  >> gain = -0.050833
		   value->CalTemp=21.0;
		   value->CalAdcValue= 16*2510; // calculated 
 	   
		   value->Beta = 0 ;
		
	  break;
	}

      Tch->setCalValues(value);
      delete value; 
      return 0;
    }
                            
int  GetHvCurrentCalParameter(  CurrentConversion* ch  )
  // ch  the HV  channel 
  //  the structure to store the calibration values
  // returns 0 if o.k.
  // should look up the ch  and return the values 
 
    { CurrentConvDef::CurrentCalValues* value = new CurrentConvDef::CurrentCalValues; 
      int ChId = ch->getId();
     
      switch (ChId)
	{ case 10 :  //
	         
	          value->Gain=   1 ; // a
	          value->CalCurrent=1e-6;
                  value->CalAdcValue=16*1622; 
		 
		  
	   break;
 
	   default:  
	                 value->Gain=   1;
			 value->CalCurrent=21.0;
			 value->CalAdcValue= 16*2510; // calculated 
 	  break;
	}

      ch->setCalValues(value);
      delete value; 
      return 0;
    }


    int  GetPowerCalParameter( PowerConversion* Pch  )
    // DacChId the peltier channel DAC
    // should look up the DacChId and return the values 
    { PowerConversionDef::PowerCalValues* value = new PowerConversionDef::PowerCalValues; 
    //int DacChId = Pch->getId();
      
       value->Gain=  0.118;//  mW /count 
       // DAC -> Current gain = 5.8 mA / count
       // Heat transfer = 290W / 14.2 A =  20.4  W / A ==> 118 mW / count 
       // peltier : 290 W  @ Tc=0 , Th= 32 @ 14.2 A ( ~ 24V)
       // max 29.5 V  
       // for the moment Tc = Tw is estimated at ~ 16 C , 11 C water + heating 
       value->CalPower=0;
       value->CalDacValue=0;
       value->MaxDac = 4095;
        Pch->setCalValues(value);
	
       delete value; 
       return 0;
    }

     int  GetHumidityCalParameter( HumidityConversion* Hch  )
    // D
    // should look up the Humidity Id and return the values 
       { HumidityConversionDef::HumidityCalValues* value = new HumidityConversionDef::HumidityCalValues ; 
       //  int ChId = Hch->getId();
      
       value->Gain=   1;// first estimation ;-)
       value->CalHumidity=0;
       value->CalAdcValue=0;
      value->VRefGain = 39.1 /( 39.1 + 9.93);
       Hch->setCalValues(value);
       delete value; 
       return 0;
    }                    
 int  GetBoxCharParameter(BoxCharacteristics *BCV )
     
   { BoxCharacteristicsDef::BoxCharValues* value = 
       new BoxCharacteristicsDef::BoxCharValues ; 
      
	value->fWeigth = 10; // kg
	value->fHeatCapacity = 14.4e3 ; //  J/ C 
	value->fLeakage= 4 ; // W /C
	value->fLoopGain= 6 ; // W/C 
	value->fDiffGain = 20 ;
	value->fBoxNr = 1;
	value->fMaxDeltaTemperature =1;
	value->fMaxTemperature = 35;
	value->fMinTemperature = -30;
	value->fFullPowerMargin =2;
	BCV->setCalValues(value);
            
	return 0;
     
   }
};

#endif 
