#ifndef __AdcchannelAd4717
#define  __AdcchannelAd4717

#include "adcchannelinterface.h"
#include "ad4717devdescription.h"
#include "TShare.h"
#include "TDeviceControl.h"

/** 
    \class AdcChannelAd4717
    \author W.Beaumont
    \date May 2002
    \version 1.0


    \brief Adc implementation of AdcChannelInterface

    

    <h2> Usage </h2>
    It implements AdcChannelInterface class and make the Daq transparent to 
    the user:
    <ul>
    <li> it connects to the Main TDeviceControl and request channel
    <li> it maps the Adc Share memory corresponding part to a AdcDescription
    <li> it controls access to this AdcDescrition thru methods lockDataBuffer,
    unlockDataBuffer,getApvEventData and release.
    </ul>
    <pre>
    // Request 64 channel 

    for(int i =0;i < 64 ; i++)
    {
    
    adcc[i] = new AdcChannelAd4717(64 ,i);
    if (!adcc[i]->isInitialized()) exit(-1);
    }
    // Loop
    int lastspill =0 ; short mybuf[10000];
    while (event_to_get)
    {
     adcc[0]->lock();
     // Check new spill
     if (adcc[0]->getSpillNumber() == lastspill) 
        adcc[0]->unlock();  
     // Spill number
     lastspill = adcc[0]->getSpillNumber();
     // Get events
     for (int j=0;j<8;j++)
       {
         adcc[i]->getAdcData(mybuf,1024);
         ....
         // do what you want with mybuf
         ....
       }
       
     adcc[0]->unlock();
    }

    // Release
    for (int i=0;i<64;i++)
     adcc[i]->release();
    </pre>



*/

class AdcChannelAd4717 : public AdcChannelInterface
{ 

  //variable 
protected: 
  /** Connect to the Daq and check if channel is available,
      it turns isInitialized() to true if Ok
  */
  virtual int  AttachADCChannel( int serialnr, int channelnr);
public:
  /** Constructor
      @param devtype the i2c controler device type to which the adc is conneted
      @param devicenr the device number of the i2c controller device the adc is connected to
      @param devchannel if the dev has more than one i2c controller the channel (else 0)
      @param adcaddress the i2c address of the adc   
      @param channel  wich channel of the adc will be used ( 1 to 4)
       */   
  AdcChannelAd4717(int  boardnr, int  channel ) ; 
   virtual ~AdcChannelAd4717();

  /** Copy Adc Data to user buffer 
      @param out_buf Output buffer
      @param size_of_buf Size of output buffer
   */   
  virtual int getAdcData(unsigned short *out_buf, int size_of_buf );
 /** Return Sample i */
  virtual unsigned short getAdcSample(int i);
  /** Return Sample i from the whole buffer */
  virtual unsigned short getSampleFromBuffer(int i);
  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber(){return fDev->getSpill();} 
  /** Lock share memory access */
  virtual void  lock() {fMemoryClient->Lock();}
  /** Unlock share memory access */
  virtual void  unlock() {fMemoryClient->Unlock();} 
  /** detach share memory access and release daq channel */
  virtual void  release();
  /** give the average ADC value for the channel 
      returns the number of samples used to calculate the avarage 
      @param adcvalue the average value of the ADC input 
  */ 
  virtual int getAdcData(float& adcvalue );
  /** only for debug reasons */ 
  virtual void   info(std::ostream& sout);


private :

  ChannelDescription* fCd;
  pid_t fPid;
  TShare *fMemoryClient;
  Ad4717Description *fAdc;
  Ad4717DevDescription *fDev; 



};

#endif 
	

